/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef IMPORTXPLANE900DATA_H_
#define IMPORTXPLANE900DATA_H_

#include <iostream>
#include <fstream>
#include <string>
#include <time.h>
#include <sys/time.h>
#include <gtkmm/window.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/button.h>
#include <libglademm.h>
#ifdef HAVE_CONFIG_H
 #include "../config.h"
#endif
#ifdef HAVE_LIBZIP
 #include <zip.h>
#else
 struct zip {};	/* An empty definition to allow the compiler to proceed */
#endif

#include "Config.h"
#include "DataStore.h"

class ImportXPlane900Data
{
public:
	enum RowCodeEnum {
		ROWCODE_INVALID=0,
		ROWCODE_AIRPORT=1,
		ROWCODE_NDB=2,
		ROWCODE_VOR=3,
		ROWCODE_LOCALIZER_ILS=4,
		ROWCODE_LOCALIZER_ONLY=5,
		ROWCODE_GLIDESLOPE=6,
		ROWCODE_OUTERMARKER=7,
		ROWCODE_MIDDLEMARKER=8,
		ROWCODE_INNERMARKER=9,
		ROWCODE_DME=12,
		ROWCODE_DME_STANDALONE=13,
		ROWCODE_VIEWPOINT=14,
		ROWCODE_STARTUPLOC=15,
		ROWCODE_SEAPORT=16,
		ROWCODE_HELIPORT=17,
		ROWCODE_ROTBEACON=18,
		ROWCODE_WINDSOCK=19,
		ROWCODE_SIGN=20,
		ROWCODE_LIGHTING=21,
		ROWCODE_FREQ_WX=50,
		ROWCODE_FREQ_UNICOM=51,
		ROWCODE_FREQ_CLD=52,
		ROWCODE_FREQ_GROUND=53,
		ROWCODE_FREQ_TOWER=54,
		ROWCODE_FREQ_APP=55,
		ROWCODE_FREQ_DEP=56,
		ROWCODE_ENDFILE=99,
		ROWCODE_RUNWAY=100,
		ROWCODE_RUNWAY_WATER=101,
		ROWCODE_HELIPAD=102,
		ROWCODE_PAVEMENT=110,
		ROWCODE_LINEARFEATURES=120,
		ROWCODE_BOUNDARY=130,
		ROWCODE_NODE=111,
		ROWCODE_NODE_BEZIER=112,
		ROWCODE_NODE_CLOSE=113,
		ROWCODE_NODE_BEZIER_CLOSE=114,
		ROWCODE_NODE_TERMINATE=115,
		ROWCODE_NODE_BEZIER_TERMINATE=116
	};

	ImportXPlane900Data(Config *Cfg, std::string *GladeFilename, DataStore *Data);
	~ImportXPlane900Data();

	int Run();
	bool ConfirmData(std::string *VolName,time_t EffFrom,time_t EffTo);

private:
	bool CheckMetaData(std::string *Line, int FormatCode, std::string *Volume);
	enum RowCodeEnum GetRowCode(std::string *Line);
	void on_CancelButton_clicked();
	void ScheduleMainLoop();
	bool GetDataLine(std::string *Line);
	bool ImportAirportData(struct zip *zp, std::string *Volume);
	bool ImportNavData(struct zip *zp, std::string *Volume);
	bool ImportFixData(struct zip *zp, std::string *Volume);

	static const double ParseAirportsProgressFrac=0.4;
	static const double ParseNavProgressFrac=0.3;
	static const double ParseFixProgressFrac=0.3;

	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Config *Cfg;
	std::string *GladeFilename;
	Gtk::Window *ProgressWindow;
	Gtk::ProgressBar *ProgressBar,*SubProgressBar;
	Gtk::Button *CancelButton;
	int BytesRead,BytesAvail;
	std::ifstream IFStream;
	struct timeval LastProgressBarUpdate;
	bool CancelFlag;
	DataStore *Data;
	Volume *VolData;
	double ProgressDone;
	unsigned char DataBuf[1024*1024];
	ssize_t DataBufLen;
	ssize_t DataBufOfs;
};

#endif /*IMPORTXPLANE900DATA_H_*/
