/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef WAYPOINTSEARCHDIALOG_H_
#define WAYPOINTSEARCHDIALOG_H_
#include <stdint.h>
#include <vector>
#include <gtkmm/dialog.h>
#include <gtkmm/button.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/entry.h>
#include <libglademm.h>
#include "Waypoint.h"
#include "DataStore.h"
#include "OnScreenKeypad.h"

class WaypointSearchDialogModelColumns : public Gtk::TreeModelColumnRecord
{
public:
	WaypointSearchDialogModelColumns();

	Gtk::TreeModelColumn<std::string> ID;
	Gtk::TreeModelColumn<std::string> LocType;
	Gtk::TreeModelColumn<std::string> Name;
	Gtk::TreeModelColumn<Waypoint*> WaypointData;
};

class WaypointSearchDialog
{
public:
	WaypointSearchDialog(std::string *GladeFilename, DataStore *Data);
	~WaypointSearchDialog();
	int Run();
	Waypoint* GetSelection();

private:
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Dialog *GtkDialog;
	Gtk::TreeView *WaypointsTreeView;
	Gtk::Entry *NameEntry;
	Gtk::Button *OKButton;
	Gtk::Button *CancelButton;
	Gtk::VBox *OnScreenKeypadParent;
	OnScreenKeypad *Keypad;

	std::vector<Waypoint*> SearchList;

	WaypointSearchDialogModelColumns Columns;
	Glib::RefPtr<Gtk::ListStore> ListStore;

	DataStore *Data;
	Waypoint* Selection;

	void on_OKButton_clicked();
	void on_CancelButton_clicked();
	void on_NameEntry_changed();
	void on_Row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column);
};

#endif /*WAYPOINTSEARCHDIALOG_H_*/
